/**********************************************************************\
    Copyright (c) 2010 Corporation for National Research Initiatives;
                        All rights reserved.
     The CNRI open source license for this software is available at 
                  http://hdl.handle.net/1895.26/1014
\**********************************************************************/

package net.cnri.apps.dogui.view;

import net.cnri.do_api.*;
import net.cnri.apps.dogui.controller.*;
import net.cnri.dobj.*;
import net.cnri.awt.*;
import net.cnri.guiutil.GridC;
import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import javax.swing.*;
import java.io.*;

public class MainWindow
  extends DOFrame
  implements ActionListener
{
  public static final String CONTENT_ELEMENT_ID = "content";
  
  private MainWindow thisObj;
  
  private JTextField entryField;
  
  private JLabel entryLabel;
  private JLabel authLabel;
  private JLabel statusField;

  private JPanel syncPanel;
  
  private JButton findButton;
  private JButton depositButton;
  
  private JLabel depositTarget;
  private DepositTransferHandler xfrHandler = null;
  
  private DNARetriever retriever = null;
  private DNAPusher pusher = null;
  
  private SyncStatusView inboxStatus = null;
  private SyncStatusView outboxStatus = null;
  private SyncStatusView indexStatus = null;
  
  private HDLAction createFromFileAction;
  
  public MainWindow(AdminToolUI ui) {
    super(ui.getStr("main_win_title"), ui);
    this.thisObj = this;
    
    //depositTarget = new JLabel(ui.getStr("deposit_target_label"));
    //depositTarget.setTransferHandler(xfrHandler = new DepositTransferHandler());
    //depositTarget.setIcon(ui.getImages().getIcon(DOImages.DEPOSIT_WELL));
    //depositTarget.addMouseListener(new DragMouseAdapter());
    
    entryLabel = new JLabel(ui.getStr("search")+": ", JLabel.RIGHT);
    entryField = new JTextField("", 35);
    findButton = new JButton(ui.getStr("find_objects"));
    findButton.setOpaque(false);
    depositButton = new JButton(ui.getStr("deposit"));
    depositButton.setOpaque(false);
    authLabel = new JLabel(" ");
    statusField = new JLabel(" ");
    statusField.setForeground(Color.gray);
    JLabel msgLabel = new JLabel(ui.getStr("drag_obj_dep_msg"), JLabel.CENTER);
    msgLabel.setForeground(Color.gray);
    
    syncPanel = new JPanel(new GridBagLayout());
    syncPanel.setOpaque(false);
    
    JPanel p = new JPanel(new GridBagLayout());
    p.setOpaque(true);
    p.setBackground(Color.white);
    JPanel p2;
    
    p2 = new JPanel(new GridBagLayout());
    p2.setOpaque(false);
    p2.add(entryLabel, GridC.getc().xy(0,0).fillx());
    p2.add(entryField, GridC.getc().xy(1,0).wx(1).fillx().insets(0,10,0,10));
    p2.add(findButton, GridC.getc().xy(2,0));
    p2.add(depositButton, GridC.getc().xy(3,0));
    p.add(p2, GridC.getc().xy(0,2).wx(1).fillx().insets(10,10,10,10));
    
    p.add(msgLabel, GridC.getc().xy(0,3).wxy(1,1).fillboth());
    
    retriever = new DNARetriever(ui);
    retriever.setThreadName("DNA Retriever");
    
    pusher =  new DNAPusher(ui);
    pusher.setThreadName("DNA Pusher");
    
    SyncStatusView pullView = new SyncStatusView(ui, retriever, ui.getStr("sync_retrieval_label"));
    pullView.setShowProgress(false);
    
    SyncStatusView pushView = new SyncStatusView(ui, pusher, ui.getStr("sync_publish_label"));
    pushView.setShowProgress(false);
    
    int x = 0;
    JPanel btnP = new JPanel(new GridBagLayout());
    btnP.setOpaque(false);
    btnP.add(pullView, GridC.getc(x++,0).insets(10,10,10,10));
    btnP.add(pushView, GridC.getc(x++,0).insets(10,0,10,10));
    btnP.add(statusField, GridC.getc(x++,0).wx(1).insets(10,0,10,10));
    btnP.add(authLabel, GridC.getc(x++,0).insets(10,0,10,10));
    btnP.add(Box.createHorizontalStrut(20), GridC.getc(x++,0));
    p.add(btnP, GridC.getc().xy(0,4).wx(1).fillx());
    
    p.setTransferHandler(xfrHandler = new DepositTransferHandler());

    getContentPane().add(p, BorderLayout.CENTER);
    getRootPane().setDefaultButton(findButton);
    
    findButton.addActionListener(this);
    depositButton.addActionListener(this);
    entryField.addActionListener(this);
    
    pack();
    setSize(getPreferredSize());
    AwtUtil.setWindowPosition(this, AwtUtil.WINDOW_CENTER);
    
    retriever.start();
    pusher.start();
    
    setDefaultCloseOperation(EXIT_ON_CLOSE);
  }
  
  public class DragMouseAdapter extends MouseAdapter {
    public void mousePressed(MouseEvent e) {
      JComponent c = (JComponent)e.getSource();
      TransferHandler handler = c.getTransferHandler();
      if(handler==null) return;
      handler.exportAsDrag(depositTarget, e, TransferHandler.COPY);
    }
  }
  

  private static final String replaceAll(String str, String toReplace, String replaceWith) {
    int toReplaceLen = toReplace.length();
    int i;
    int lastI = 0;
    StringBuffer sb = new StringBuffer(str.length());
    while((i=str.indexOf(toReplace, lastI))>=0) {
      sb.append(str.substring(lastI, i)); // add the text before the matched substring
      sb.append(replaceWith);
      lastI = i + toReplace.length();
    }
    sb.append(str.substring(lastI)); // add the rest
    return sb.toString();
  }

  
  public void setVisible(boolean val) {
    super.setVisible(val);
    entryField.requestFocus();
  }
  
  
  
  public void authenticationChanged(DOAuthentication newAuth) {
    if(newAuth==null) {
      authLabel.setText(" ");
    } else {
      authLabel.setText(String.valueOf(newAuth));
    }
  }
  
  public boolean createObject(String id) {
    Repository repo = appUI.getRepository(AdminToolUI.OUTBOX);
    
    if(repo==null) return false;
    
    try {
      // create the object on the given repository
      appUI.showErrorMessage(this, "Create-object operation not implemented yet ");
    } catch (Exception e) {
      appUI.showErrorMessage(this, "Error creating object: '"+id+"'\n\nMessage: "+e);
      return false;
    }
    return true;
  }
  
  
  
  public boolean viewObject(String objectID) {
    Repository outbox = appUI.getRepository(AdminToolUI.OUTBOX);
    DigitalObject object = null;
    if(outbox!=null) {
      try {
        object = outbox.getDigitalObject(objectID);
      } catch (Exception e) {
        System.err.println("Error checking for object in outbox: "+e);
      }
    }
    
    if(object==null) {
      // it is probably in the inbox
      Repository inbox = appUI.getRepository(AdminToolUI.INBOX);
      if(inbox!=null) {
        try {
          object = inbox.getDigitalObject(objectID);
        } catch (Exception e) {
          System.err.println("Error checking for object in outbox: "+e);
        }
      }
    }
    
    if(object==null) {
      appUI.showErrorMessage(this, replaceAll(appUI.getStr("obj_not_found"),
                                              "{objectid}", objectID));
      return false;
    }
    
    return viewObject(object);
  }
  
  public boolean viewObject(DigitalObject object) {
    if(object==null) return false;
    SwingUtilities.invokeLater(new ObjectViewer(object));
    return true;
  }
  
  private class ObjectViewer
    implements Runnable
  {
    private DigitalObject object;
    
    ObjectViewer(DigitalObject obj) {
      this.object = obj;
    }
    
    public void run() {
      ShowObjectWindow sow = new ShowObjectWindow(appUI);
      sow.setObject(object, ShowObjectWindow.VIEW_MODE);
      sow.setVisible(true);
    }
  }
  
  class SearchResult {
    String objectID;
    String repoID;
    String label;
    String source;

    SearchResult(HeaderSet searchResult, String source) {
      objectID = searchResult.getStringHeader("objectid", "");
      repoID = searchResult.getStringHeader("repoid", null);
      label = searchResult.getStringHeader("desc", null);
      this.source = source;
    }
    
    public String toString() {
      StringBuffer sb = new StringBuffer();
      if(source!=null) {
        sb.append(source);
        sb.append(": ");
      }
      sb.append(String.valueOf(objectID));
      if(repoID!=null) {
        sb.append(" @ ");
        sb.append(repoID);
      }
      if(label!=null) {
        sb.append(":  ");
        sb.append(label);
      }
      return sb.toString();
    }
  }
  
  
  public void deleteObject(String repoID, String id) {
    if(id==null || id.trim().length()<=0) return;
    
    try {
      appUI.showErrorMessage(this, "Delete-object operation not implemented yet ");
    } catch (Exception e) {
      appUI.showErrorMessage(this, "There was an error deleting object '"+id+"': "+e);
    }
  }
  
  /** Returns true if the given string looks like a handle */
  public static final boolean looksLikeID(String id) {
    if(id==null) return false;
    int slashIdx = id.indexOf('/');
    if(slashIdx<0) return false; // no slash
    if(id.indexOf(' ')>=0) return false; // no space
    return true;
  }
  
  public void actionPerformed(ActionEvent evt) {
    Object src = evt.getSource();
    if(src==entryField || src==findButton) {
      String query = entryField.getText().trim();
      if(query.length()<=0) {
        appUI.showErrorMessage(this, appUI.getStr("empty_search_err_msg"));
        return;
      }
      
      //if(looksLikeID(query)) {
      //  viewObject(query);
      //} else {
        SearchWindow searchWin = new SearchWindow(appUI);
        searchWin.setVisible(true);
        searchWin.performSearch(new DOSearch(query));
      //}
    } else if(src==depositButton) {
      FileDialog fwin = new FileDialog(this, appUI.getStr("choose_deposit_file"), 
                                       FileDialog.LOAD);
      fwin.setVisible(true);
      String fileStr = fwin.getFile();
      String dirStr = fwin.getDirectory();
      if(fileStr==null || dirStr==null) return;
      File f[] = new File[] { new File(dirStr, fileStr) };
      PreDepositWindow depWin = new PreDepositWindow(appUI, f);
      depWin.setVisible(true);
    }
  }
  
  
  
  private class DepositTransferHandler 
    extends TransferHandler
  {
    private DataFlavor fileFlavor;
    
    public DepositTransferHandler() {
      super("text");
      fileFlavor = DataFlavor.javaFileListFlavor;
    }
    
    public boolean canImport(JComponent c, DataFlavor[] flavors) {
      if (hasFileFlavor(flavors))   { return true; }
      return false;
    }
    private boolean hasFileFlavor(DataFlavor[] flavors) {
      for (int i=0; i<flavors.length; i++) {
        if (DataFlavor.javaFileListFlavor.equals(flavors[i])) {
          return true;
        }
      }
      return false;
    }
    
    public int getSourceActions(JComponent c) { return COPY; }
    
    
    /** Import the indicated transferable into the digital object */
    public boolean importData(JComponent c, Transferable t) {
      try {
        JLabel tc;
        System.err.println("trying import: "+t);
        if(!canImport(c, t.getTransferDataFlavors())) {
          return false;
        }
        
        if (hasFileFlavor(t.getTransferDataFlavors())) {
          System.err.println("importing file list...");
          java.util.List files = 
            (java.util.List)t.getTransferData(DataFlavor.javaFileListFlavor);
          System.err.println("importing file list: "+files);
          
          File fileArray[] = (File[])files.toArray(new File[files.size()]);
          PreDepositWindow win = new PreDepositWindow(appUI, fileArray);
          win.setVisible(true);
          return true;
        }
      } catch (Exception e) {
        System.err.println("Error importing data: "+e);
        e.printStackTrace(System.err);
      }
      return false;
    }
  }
  
  /*
  private class ShowObjectRunner
    implements Runnable
  {
    private String repoID;
    private String objectID;
    private int mode;
    
    ShowObjectRunner(String repoID, String objectID, int mode) {
      this.repoID = repoID;
      this.objectID = objectID;
      this.mode = mode;
    }
    
    public void run() {
      ShowObjectWindow showObjectWindow = new ShowObjectWindow(ui);
      showObjectWindow.setObject(repoID, objectID, mode);
      AwtUtil.setWindowPosition(showObjectWindow, thisObj);
      showObjectWindow.setVisible(true);
    }
  }
*/
  
}
